@@ -16,7 +16,7 @@ from bitarray._bitarray import (bitarray, decodetree, _sysinfo,
                                 __version__)
 
 
-__all__ = ['bitarray', 'frozenbitarray', 'decodetree', '__version__']
+__all__ = ['bitarray', 'frozenbitarray', 'decodetree', 'bits2bytes']
 
 
 class frozenbitarray(bitarray):
@@ -24,9 +24,8 @@ class frozenbitarray(bitarray):
 frozenbitarray
 
 Return a frozenbitarray object, which is initialized the same way a bitarray
-object is initialized.  A frozenbitarray is immutable and hashable.
-Its contents cannot be altered after it is created; however, it can be used
-as a dictionary key.
+object is initialized.  A frozenbitarray is immutable and hashable,
+and (unlike a bitarray) may be used as a dictionary key.
 """
     def __init__(self, *args, **kwargs):
         self._freeze()
