@@ -354,9 +354,9 @@ PyDoc_STRVAR(subset_doc,
 "subset(a, b, /) -> bool\n\
 \n\
 Return `True` if bitarray `a` is a subset of bitarray `b`.\n\
-`subset(a, b)` is equivalent to `(a & b).count() == a.count()` but is more\n\
-efficient since we can stop as soon as one mismatch is found, and no\n\
-intermediate bitarray object gets created.");
+`subset(a, b)` is equivalent to `a | b == b` (and `a & b == a`) but more\n\
+efficient as iterating the buffers can be stopped as soon as one mismatch\n\
+is found.  Moreover, no intermediate bitarray object is created.");
 
 /* ---------------------------- serialization -------------------------- */
 
@@ -423,7 +423,9 @@ ba2hex(PyObject *module, PyObject *a)
         return NULL;
     }
 
-    /* strsize = aa->nbits / 4;  could make strsize odd */
+    /* We want strsize to be even, such that we can transform the entire
+       bitarray buffer at once.  Hence, we don't use aa->nbits / 4 here, as
+       is could make strsize odd. */
     strsize = 2 * Py_SIZE(a);
     str = (char *) PyMem_Malloc(strsize);
     if (str == NULL)
@@ -612,10 +614,10 @@ For `n=32` the RFC 4648 Base32 alphabet is used, and for `n=64` the\n\
 standard base 64 alphabet is used.");
 
 
-/* Translate ASCII digits into the bitarray's buffer.
+/* Translate ASCII digits into bitarray.
    The (Python) arguments to this functions are:
    - base n, one of 2, 4, 8, 16, 32, 64  (n=2^m   where m bits per digit)
-   - bitarray (of length m * len(s)) whose buffer is written into
+   - bitarray (of length m * len(s)) whose elements are overwritten
    - byte object s containing the ASCII digits
 */
 static PyObject *
@@ -631,7 +633,7 @@ base2ba(PyObject *module, PyObject *args)
         return NULL;
     if ((m = base_to_length(n)) < 0)
         return NULL;
-    if (a == NULL)
+    if (a == NULL)  /* when only base n is given - return length log2(n) */
         return PyLong_FromLong(m);
 
 #define aa  ((bitarrayobject *) a)
