@@ -4,7 +4,7 @@
 
    Author: Ilan Schnell
 */
-#define BITARRAY_VERSION  "2.5.0"
+#define BITARRAY_VERSION  "2.6.1"
 
 #ifdef STDC_HEADERS
 #include <stddef.h>
@@ -69,6 +69,9 @@ typedef struct {
 /* the endianness string */
 #define ENDIAN_STR(endian)  ((endian) == ENDIAN_LITTLE ? "little" : "big")
 
+/* number of pad bits */
+#define PADBITS(self)  (8 * Py_SIZE(self) - (self)->nbits)
+
 /* number of bytes necessary to store given bits */
 #define BYTES(bits)  (((bits) + 7) >> 3)
 
@@ -129,21 +132,20 @@ zeroed_last_byte(bitarrayobject *self)
     const int r = self->nbits % 8;     /* index into mask table */
 
     assert(r > 0);
-    assert_nbits(self);
-    return ones_table[IS_BE(self)][r] & self->ob_item[Py_SIZE(self) - 1];
+    return self->ob_item[Py_SIZE(self) - 1] & ones_table[IS_BE(self)][r];
 }
 
 /* Unless buffer is readonly, zero out pad bits.
    Always return the number of pad bits - leave self->nbits unchanged */
 static inline int
-setunused(bitarrayobject *self)
+set_padbits(bitarrayobject *self)
 {
-    const int r = self->nbits % 8;
+    const int r = self->nbits % 8;     /* index into mask table */
 
     if (r == 0)
         return 0;
     if (self->readonly == 0)
-        self->ob_item[Py_SIZE(self) - 1] = zeroed_last_byte(self);
+        self->ob_item[Py_SIZE(self) - 1] &= ones_table[IS_BE(self)][r];
     return 8 - r;
 }
 
@@ -217,20 +219,21 @@ adjust_step_positive(Py_ssize_t slicelength,
     assert(*step != 1 || slicelength == 0 || *stop - *start == slicelength);
 }
 
-/* Interpret a PyObject (usually PyLong or PyBool) as a bit, return 0 or 1.
-   On error, return -1 and set error message. */
+/* convert Python object to C int and set value at address -
+   return 1 on success, 0 on failure (and set exception) */
 static inline int
-pybit_as_int(PyObject *value)
+conv_pybit(PyObject *value, int *vi)
 {
-    Py_ssize_t x;
+    Py_ssize_t n;
 
-    x = PyNumber_AsSsize_t(value, NULL);
-    if (x == -1 && PyErr_Occurred())
-        return -1;
+    n = PyNumber_AsSsize_t(value, NULL);
+    if (n == -1 && PyErr_Occurred())
+        return 0;
 
-    if (x < 0 || x > 1) {
-        PyErr_Format(PyExc_ValueError, "bit must be 0 or 1, got %zd", x);
-        return -1;
+    if (n < 0 || n > 1) {
+        PyErr_Format(PyExc_ValueError, "bit must be 0 or 1, got %zd", n);
+        return 0;
     }
-    return (int) x;
+    *vi = (int) n;
+    return 1;
 }
