@@ -10,26 +10,23 @@ from __future__ import absolute_import
 import os
 import sys
 
-from bitarray import bitarray, bits2bytes, get_default_endian
+from bitarray import bitarray, bits2bytes
 
 from bitarray._util import (
     count_n, rindex, parity, count_and, count_or, count_xor, subset,
     serialize, ba2hex, _hex2ba, ba2base, _base2ba, vl_encode, _vl_decode,
-    _set_bato,
+    canonical_decode,
 )
 
 __all__ = [
     'zeros', 'urandom', 'pprint', 'make_endian', 'rindex', 'strip', 'count_n',
     'parity', 'count_and', 'count_or', 'count_xor', 'subset',
     'ba2hex', 'hex2ba', 'ba2base', 'base2ba', 'ba2int', 'int2ba',
-    'serialize', 'deserialize', 'vl_encode', 'vl_decode', 'huffman_code',
+    'serialize', 'deserialize', 'vl_encode', 'vl_decode',
+    'huffman_code', 'canonical_huffman', 'canonical_decode',
 ]
 
 
-# tell the _util extension what the bitarray type object is, such that it
-# can check for instances thereof
-_set_bato(bitarray)
-
 _is_py2 = bool(sys.version_info[0] == 2)
 
 
@@ -42,7 +39,7 @@ endianness, which may be 'big', 'little'.
     if not isinstance(__length, (int, long) if _is_py2 else int):
         raise TypeError("int expected, got '%s'" % type(__length).__name__)
 
-    a = bitarray(__length, get_default_endian() if endian is None else endian)
+    a = bitarray(__length, endian)
     a.setall(0)
     return a
 
@@ -52,7 +49,7 @@ def urandom(__length, endian=None):
 
 Return a bitarray of `length` random bits (uses `os.urandom`).
 """
-    a = bitarray(0, get_default_endian() if endian is None else endian)
+    a = bitarray(0, endian)
     a.frombytes(os.urandom(bits2bytes(__length)))
     del a[__length:]
     return a
@@ -138,28 +135,28 @@ def strip(__a, mode='right'):
 Return a new bitarray with zeros stripped from left, right or both ends.
 Allowed values for mode are the strings: `left`, `right`, `both`
 """
-    if not isinstance(__a, bitarray):
-        raise TypeError("bitarray expected, got '%s'" % type(__a).__name__)
     if not isinstance(mode, str):
         raise TypeError("str expected for mode, got '%s'" % type(__a).__name__)
     if mode not in ('left', 'right', 'both'):
         raise ValueError("mode must be 'left', 'right' or 'both', got %r" %
                          mode)
-    first = 0
-    if mode in ('left', 'both'):
+    if mode == 'right':
+        start = None
+    else:
         try:
-            first = __a.index(1)
+            start = __a.index(1)
         except ValueError:
             return __a[:0]
 
-    last = len(__a) - 1
-    if mode in ('right', 'both'):
+    if mode == 'left':
+        stop = None
+    else:
         try:
-            last = rindex(__a)
+            stop = rindex(__a) + 1
         except ValueError:
             return __a[:0]
 
-    return __a[first:last + 1]
+    return __a[start:stop]
 
 
 def hex2ba(__s, endian=None):
@@ -173,8 +170,7 @@ Bitarray of hexadecimal representation.  hexstr may contain any number
     if not isinstance(__s, bytes):
         raise TypeError("str expected, got '%s'" % type(__s).__name__)
 
-    a = bitarray(4 * len(__s),
-                 get_default_endian() if endian is None else endian)
+    a = bitarray(4 * len(__s), endian)
     _hex2ba(a, __s)
     return a
 
@@ -194,8 +190,7 @@ standard base 64 alphabet is used.
     if not isinstance(__s, bytes):
         raise TypeError("str expected, got '%s'" % type(__s).__name__)
 
-    a = bitarray(_base2ba(__n) * len(__s),
-                 get_default_endian() if endian is None else endian)
+    a = bitarray(_base2ba(__n) * len(__s), endian)
     _base2ba(__n, a, __s)
     return a
 
@@ -214,8 +209,8 @@ The bit-endianness of the bitarray is respected.
         raise ValueError("non-empty bitarray expected")
 
     le = bool(__a.endian() == 'little')
-    if length % 8:
-        pad = zeros(8 - length % 8, __a.endian())
+    if __a.padbits:
+        pad = zeros(__a.padbits, __a.endian())
         __a = __a + pad if le else pad + __a
 
     if _is_py2:
@@ -269,7 +264,7 @@ and requires `length` to be provided.
             raise OverflowError("unsigned integer not in range(0, %d), "
                                 "got %d" % (1 << length, __i))
 
-    a = bitarray(0, get_default_endian() if endian is None else endian)
+    a = bitarray(0, endian)
     le = bool(a.endian() == 'little')
     if _is_py2:
         s = hex(__i)[2:].rstrip('L')
@@ -287,7 +282,7 @@ and requires `length` to be provided.
     if la > length:
         a = a[:length] if le else a[-length:]
     if la < length:
-        pad = zeros(length - la, endian)
+        pad = zeros(length - la, a.endian())
         a = a + pad if le else pad + a
     assert len(a) == length
     return a
@@ -296,36 +291,79 @@ and requires `length` to be provided.
 def deserialize(__b):
     """deserialize(bytes, /) -> bitarray
 
-Return a bitarray given the bytes representation returned by `serialize()`.
+Return a bitarray given a bytes-like representation such as returned
+by `serialize()`.
 """
+    if isinstance(__b, int):  # as bytes(n) will return n NUL bytes
+        raise TypeError("cannot convert 'int' object to bytes")
     if not isinstance(__b, bytes):
-        raise TypeError("bytes expected, got '%s'" % type(__b).__name__)
+        __b = bytes(__b)
     if len(__b) == 0:
         raise ValueError("non-empty bytes expected")
 
-    head = ord(__b[0]) if _is_py2 else __b[0]
-    assert isinstance(head, int)
-    if head >= 32 or head % 16 >= 8:
-        raise ValueError('invalid header byte 0x%02x' % head)
-    return bitarray(__b)
+    if _is_py2:
+        head = ord(__b[0])
+        if head >= 32 or head % 16 >= 8:
+            raise ValueError('invalid header byte: 0x%02x' % head)
+    try:
+        return bitarray(__b)
+    except TypeError:
+        raise ValueError('invalid header byte: 0x%02x' % __b[0])
 
 
 def vl_decode(__stream, endian=None):
     """vl_decode(stream, /, endian=None) -> bitarray
 
-Decode binary stream (an integer iterator, or bytes object), and return
+Decode binary stream (an integer iterator, or bytes-like object), and return
 the decoded bitarray.  This function consumes only one bitarray and leaves
 the remaining stream untouched.  `StopIteration` is raised when no
 terminating byte is found.
 Use `vl_encode()` for encoding.
 """
-    if isinstance(__stream, bytes):
-        __stream = iter(__stream)
-
-    a = bitarray(32, get_default_endian() if endian is None else endian)
-    _vl_decode(__stream, a)
+    a = bitarray(32, endian)
+    _vl_decode(iter(__stream), a)
     return a
 
+# ------------------------------ Huffman coding -----------------------------
+
+def _huffman_tree(__freq_map):
+    """_huffman_tree(dict, /) -> Node
+
+Given a dict mapping symbols to their frequency, construct a Huffman tree
+and return its root node.
+"""
+    from heapq import heappush, heappop
+
+    class Node(object):
+        """
+        A Node instance will either have a 'symbol' (leaf node) or
+        a 'child' (a tuple with both children) attribute.
+        The 'freq' attribute will always be present.
+        """
+        def __lt__(self, other):
+            # heapq needs to be able to compare the nodes
+            return self.freq < other.freq
+
+    minheap = []
+    # create all leaf nodes and push them onto the queue
+    for sym, f in __freq_map.items():
+        leaf = Node()
+        leaf.symbol = sym
+        leaf.freq = f
+        heappush(minheap, leaf)
+
+    # repeat the process until only one node remains
+    while len(minheap) > 1:
+        # take the two nodes with lowest frequencies from the queue
+        # to construct a new node and push it onto the queue
+        parent = Node()
+        parent.child = heappop(minheap), heappop(minheap)
+        parent.freq = parent.child[0].freq + parent.child[1].freq
+        heappush(minheap, parent)
+
+    # the single remaining node is the root of the Huffman tree
+    return minheap[0]
+
 
 def huffman_code(__freq_map, endian=None):
     """huffman_code(dict, /, endian=None) -> dict
@@ -335,47 +373,22 @@ calculate the Huffman code, i.e. a dict mapping those symbols to
 bitarrays (with given endianness).  Note that the symbols are not limited
 to being strings.  Symbols may may be any hashable object (such as `None`).
 """
-    from heapq import heappush, heappop
-
     if not isinstance(__freq_map, dict):
         raise TypeError("dict expected, got '%s'" % type(__freq_map).__name__)
-    if len(__freq_map) == 0:
-        raise ValueError("non-empty dict expected")
-    if endian is None:
-        endian = get_default_endian()
 
-    class Node(object):
-        # a Node object will have either .symbol or .child set below,
-        # .freq will always be set
-        def __lt__(self, other):
-            # heapq needs to be able to compare the nodes
-            return self.freq < other.freq
+    b0 = bitarray('0', endian)
+    b1 = bitarray('1', endian)
 
-    def huff_tree(freq_map):
-        # given a dictionary mapping symbols to thier frequency,
-        # construct a Huffman tree and return its root node
-
-        minheap = []
-        # create all leaf nodes and push them onto the queue
-        for sym, f in freq_map.items():
-            nd = Node()
-            nd.symbol = sym
-            nd.freq = f
-            heappush(minheap, nd)
-
-        # repeat the process until only one node remains
-        while len(minheap) > 1:
-            # take the two nodes with smallest frequencies from the queue
-            child_0 = heappop(minheap)
-            child_1 = heappop(minheap)
-            # construct the new internal node and push it onto the queue
-            parent = Node()
-            parent.child = [child_0, child_1]
-            parent.freq = child_0.freq + child_1.freq
-            heappush(minheap, parent)
-
-        # the single remaining node is the root of the Huffman tree
-        return minheap[0]
+    if len(__freq_map) < 2:
+        if len(__freq_map) == 0:
+            raise ValueError("cannot create Huffman code with no symbols")
+        # Only one symbol: Normally if only one symbol is given, the code
+        # could be represented with zero bits.  However here, the code should
+        # be at least one bit for the .encode() and .decode() methods to work.
+        # So we represent the symbol by a single code of length one, in
+        # particular one 0 bit.  This is an incomplete code, since if a 1 bit
+        # is received, it has no meaning and will result in an error.
+        return {list(__freq_map)[0]: b0}
 
     result = {}
 
@@ -383,8 +396,63 @@ to being strings.  Symbols may may be any hashable object (such as `None`).
         try:                    # leaf
             result[nd.symbol] = prefix
         except AttributeError:  # parent, so traverse each of the children
-            traverse(nd.child[0], prefix + bitarray('0'))
-            traverse(nd.child[1], prefix + bitarray('1'))
+            traverse(nd.child[0], prefix + b0)
+            traverse(nd.child[1], prefix + b1)
 
-    traverse(huff_tree(__freq_map))
+    traverse(_huffman_tree(__freq_map))
     return result
+
+
+def canonical_huffman(__freq_map):
+    """canonical_huffman(dict, /) -> tuple
+
+Given a frequency map, a dictionary mapping symbols to their frequency,
+calculate the canonical Huffman code.  Returns a tuple containing:
+
+0. the canonical Huffman code as a dict mapping symbols to bitarrays
+1. a list containing the number of symbols of each code length
+2. a list of symbols in canonical order
+
+Note: the two lists may be used as input for `canonical_decode()`.
+"""
+    if not isinstance(__freq_map, dict):
+        raise TypeError("dict expected, got '%s'" % type(__freq_map).__name__)
+
+    if len(__freq_map) < 2:
+        if len(__freq_map) == 0:
+            raise ValueError("cannot create Huffman code with no symbols")
+        # Only one symbol: see note above in huffman_code()
+        sym = list(__freq_map)[0]
+        return {sym: bitarray('0', 'big')}, [0, 1], [sym]
+
+    code_length = {}  # map symbols to their code length
+
+    def traverse(nd, length=0):
+        # traverse the Huffman tree, but (unlike in huffman_code() above) we
+        # now just simply record the length for reaching each symbol
+        try:                    # leaf
+            code_length[nd.symbol] = length
+        except AttributeError:  # parent, so traverse each of the children
+            traverse(nd.child[0], length + 1)
+            traverse(nd.child[1], length + 1)
+
+    traverse(_huffman_tree(__freq_map))
+
+    # we now have a mapping of symbols to their code length,
+    # which is all we need
+
+    table = sorted(code_length.items(), key=lambda item: (item[1], item[0]))
+
+    maxbits = max(item[1] for item in table)
+    codedict = {}
+    count = (maxbits + 1) * [0]
+
+    code = 0
+    for i, (sym, length) in enumerate(table):
+        codedict[sym] = int2ba(code, length, 'big')
+        count[length] += 1
+        if i + 1 < len(table):
+            code += 1
+            code <<= table[i + 1][1] - length
+
+    return codedict, count, [item[0] for item in table]
